@@ -1,16 +1,16 @@
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2007, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -18,7 +18,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: strtok.c,v 1.11 2003/01/29 10:14:24 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -27,18 +26,20 @@
 #include <stddef.h>
 #include <string.h>
 
+#include "strtok.h"
+
 char *
 Curl_strtok_r(char *ptr, const char *sep, char **end)
 {
-  if (!ptr)
+  if(!ptr)
     /* we got NULL input so then we get our last position instead */
     ptr = *end;
 
   /* pass all letters that are including in the separator string */
-  while (*ptr && strchr(sep, *ptr))
+  while(*ptr && strchr(sep, *ptr))
     ++ptr;
 
-  if (*ptr) {
+  if(*ptr) {
     /* so this is where the next piece of string starts */
     char *start = ptr;
 
@@ -47,10 +48,10 @@ Curl_strtok_r(char *ptr, const char *sep, char **end)
 
     /* scan through the string to find where it ends, it ends on a
        null byte or a character that exists in the separator string */
-    while (**end && !strchr(sep, **end))
+    while(**end && !strchr(sep, **end))
       ++*end;
 
-    if (**end) {
+    if(**end) {
       /* the end is not a null byte */
       **end = '\0';  /* zero terminate it! */
       ++*end;        /* advance the last pointer to beyond the null byte */
