@@ -1,18 +1,18 @@
-#ifndef __CURL_STRTOK_H
-#define __CURL_STRTOK_H
+#ifndef HEADER_CURL_STRTOK_H
+#define HEADER_CURL_STRTOK_H
 /***************************************************************************
- *                                  _   _ ____  _     
- *  Project                     ___| | | |  _ \| |    
- *                             / __| | | | |_) | |    
- *                            | (__| |_| |  _ <| |___ 
+ *                                  _   _ ____  _
+ *  Project                     ___| | | |  _ \| |
+ *                             / __| | | | |_) | |
+ *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
  * are also available at http://curl.haxx.se/docs/copyright.html.
- * 
+ *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
  * furnished to do so, under the terms of the COPYING file.
@@ -20,7 +20,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: strtok.h,v 1.10 2008-07-11 18:52:35 yangtse Exp $
  ***************************************************************************/
 #include "setup.h"
 #include <stddef.h>
@@ -32,5 +31,4 @@ char *Curl_strtok_r(char *s, const char *delim, char **last);
 #include <string.h>
 #endif
 
-#endif
-
+#endif /* HEADER_CURL_STRTOK_H */
