@@ -5,7 +5,6 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib537.c,v 1.21 2008-09-20 04:26:57 yangtse Exp $
  */
 
 #include "test.h"
@@ -204,7 +203,7 @@ static int rlimit(int keep_open)
   /*
    * test 537 is all about testing libcurl functionality
    * when the system has nearly exhausted the number of
-   * available file descriptors. Test 537 will try to run 
+   * available file descriptors. Test 537 will try to run
    * with a very small number of file descriptors available.
    * This implies that any file descriptor which is open
    * when the test runs will have a number in the high range
@@ -323,8 +322,8 @@ static int rlimit(int keep_open)
 
   /* create a bunch of file descriptors */
 
-  for (num_open.rlim_cur = 1; 
-       num_open.rlim_cur < num_open.rlim_max; 
+  for (num_open.rlim_cur = 1;
+       num_open.rlim_cur < num_open.rlim_max;
        num_open.rlim_cur++) {
 
     fd[num_open.rlim_cur] = dup(fd[0]);
@@ -471,7 +470,7 @@ int test(char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  /* run the test with the bunch of open file descriptors 
+  /* run the test with the bunch of open file descriptors
      and close them all once the test is over */
 
   if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
@@ -487,11 +486,13 @@ int test(char *URL)
     return TEST_ERR_MAJOR_BAD;
   }
 
-  curl_easy_setopt(curl, CURLOPT_URL, URL);
-  curl_easy_setopt(curl, CURLOPT_HEADER, 1L);
+  test_setopt(curl, CURLOPT_URL, URL);
+  test_setopt(curl, CURLOPT_HEADER, 1L);
 
   res = curl_easy_perform(curl);
 
+test_cleanup:
+
   close_file_descriptors();
   curl_easy_cleanup(curl);
   curl_global_cleanup();
