@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,6 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: resolve.c,v 1.13 2008-10-28 20:03:22 danf Exp $
  ***************************************************************************/
 
 /* Purpose
@@ -30,6 +29,9 @@
  * Like if 'localhost' actual exists etc.
  *
  */
+
+#define CURL_NO_OLDIES
+
 #include "setup.h" /* portability help from the lib directory */
 
 #ifdef HAVE_SIGNAL_H
@@ -128,8 +130,8 @@ int main(int argc, char *argv[])
   else {
 #ifdef ENABLE_IPV6
     /* Check that the system has IPv6 enabled before checking the resolver */
-    int s = socket(PF_INET6, SOCK_DGRAM, 0);
-    if(s == -1)
+    curl_socket_t s = socket(PF_INET6, SOCK_DGRAM, 0);
+    if(s == CURL_SOCKET_BAD)
       /* an ipv6 address was requested and we can't get/use one */
       rc = -1;
     else {
